import {
  Body,
  Controller,
  Get,
  Post,
  Req,
  Res,
  UnauthorizedException,
  UseGuards,
} from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { Request, Response } from 'express';
import { AuthService } from '../services/auth.service';
import { RefreshTokenDto } from '../dto/refresh-token.dto';
import { GithubAuthGuard } from '../guards/github-auth.guard';
import { JwtAuthGuard } from '../guards/jwt-auth.guard';
import { JwtRefreshGuard } from '../guards/jwt-refresh.guard';
import { GetUser } from '../decorators/get-user.decorator';

@Controller('auth')
export class AuthController {
  constructor(
    private readonly authService: AuthService,
    private readonly configService: ConfigService,
  ) {}

  /**
   * Initiate GitHub OAuth flow
   */
  @Get('github')
  @UseGuards(GithubAuthGuard)
  githubAuth() {
    // This route is handled by the GitHub strategy
    // The actual implementation is in the GithubAuthGuard
  }

  /**
   * Handle GitHub OAuth callback
   */
  @Get('github/callback')
  @UseGuards(GithubAuthGuard)
  async githubAuthCallback(@Req() req: Request, @Res() res: Response) {
    // The user is already validated by the GitHub strategy
    // and attached to the request object
    const user = req.user as any;

    if (!user) {
      throw new UnauthorizedException('Authentication failed');
    }

    // Generate tokens
    const tokens = await this.authService.generateTokens(user.id);

    // Redirect to the frontend with tokens
    const frontendUrl = this.configService.get<string>('FRONTEND_URL') || 'http://localhost:3000';
    const redirectUrl = `${frontendUrl}/auth/callback?accessToken=${tokens.accessToken}&refreshToken=${tokens.refreshToken}`;

    return res.redirect(redirectUrl);
  }

  /**
   * Refresh tokens
   */
  @Post('refresh')
  @UseGuards(JwtRefreshGuard)
  async refreshTokens(@GetUser() user) {
    return this.authService.refreshTokens(user.id, user.refreshToken);
  }

  /**
   * Get current user profile
   */
  @Get('profile')
  @UseGuards(JwtAuthGuard)
  getProfile(@GetUser() user) {
    return user;
  }
}
